@@ -8,12 +8,13 @@
 # you should call this function when you want to use this library (can be called multiple times, to change ui type or other settings)
 # $1 ui type (dia or cli). defaults to cli
 # $2 directory for tmp files. default /tmp (leave empty for default)
-# $3 logfile (or array of logfiles) (leave empty to disable logging)
+# $3 logfile (or string of logfiles, separated by whitespace) (leave empty to disable logging)
 # $4 debug categories (leave empty to disable debugging) (an array of categories you will use in debug calls. useful when grepping logfiles)
 # this library uses the UI debug category internally, you don't need to specify it. we add it automatically
 libui-sh-init ()
 {
 	LIBUI_UI=${1:-cli}
+	[ "$LIBUI_UI" == 'dialog' ] && ! which dialog &>/dev/null && die_error "Required dependency dialog not found"
 	LIBUI_TMP_DIR=/tmp
 	if [ -n "$2" ]; then
 		LIBUI_TMP_DIR=$2
@@ -21,13 +22,14 @@ libui-sh-init ()
 	LIBUI_LOG=0
 	if [ -n "$3" ]; then
 		LIBUI_LOG=1
-		LIBUI_LOG_FILE=$3
+		LIBUI_LOG_FILE="$3"
 	fi
 	LIBUI_DEBUG=0
-	if [ -n "$4" ]; then
+	shift 3
+	if [ -n "$1" ]; then
 		LIBUI_DEBUG=1
-		LIBUI_DEBUG_CATEGORIES=$4
-		LIBUI_DEBUG_CATEGORIES+=('UI')
+		LIBUI_DEBUG_CATEGORIES=("$@")
+		check_is_in 'UI' "${LIBUI_DEBUG_CATEGORIES[@]}" || LIBUI_DEBUG_CATEGORIES+=('UI')
 	fi
 	LIBUI_DIA_SUCCESSIVE_ITEMS=$LIBUI_TMP_DIR/libui-sh-dia-successive-items
 	LIBUI_FOLLOW_PID=$LIBUI_TMP_DIR/libui-sh-follow-pid
@@ -41,14 +43,12 @@ libui-sh-init ()
 check_is_in ()
 {
 	[ -z "$1" ] && die_error "check_is_in needs a non-empty needle as \$1 and a haystack as \$2!(got: check_is_in '$1' '$2'" # haystack can be empty though
-	NEEDLE=$1
-	HAYSTACK=$2
 
-	local pattern="$NEEDLE" element
+	local needle="$1" element
 	shift
 	for element
 	do
-		[[ $element = $pattern ]] && return 0
+		[[ $element = $needle ]] && return 0
 	done
 	return 1
 }
@@ -98,24 +98,25 @@ notify ()
 
 # like notify, but user does not need to confirm explicitly when in dia mode
 # $1 str
-# $2 0/<listname> this infofy call is part of a successive list of things (eg repeat previous things, keep adding items to a list) (only needed for dia, cli does this by design).
+# $2 0/<listname> this inform call is part of a successive list of things (eg repeat previous things, keep adding items to a list) (only needed for dia, cli does this by design).
 #   You can keep several "lists of successive things" by grouping them with <listname>
 #   this is somewhat similar to follow_progress.  Note that this list isn't cleared unless you set $3 to 1.  default 0. (optional).
 # $3 0/1 this is the last one of the group of several things (eg clear buffer).  default 0. (optional)
-infofy () #TODO: when using successive things, the screen can become full and you only see the old stuff, not the new
+inform () #TODO: when using successive things, the screen can become full and you only see the old stuff, not the new
 {
 	successive=${2:-0}
 	succ_last=${3:-0}
-	debug 'UI' "infofy: $1"
-	[ `type -t _${LIBUI_UI}_infofy` == function ] || die_error "_${LIBUI_UI}_infofy is not a function"
-	_${LIBUI_UI}_infofy "$1" $successive $succ_last
+	debug 'UI' "inform: $1"
+	[ `type -t _${LIBUI_UI}_inform` == function ] || die_error "_${LIBUI_UI}_inform is not a function"
+	_${LIBUI_UI}_inform "$1" $successive $succ_last
 }
 
 # logging of stuff
 log ()
 {
 	[ "$LIBUI_LOG" = 1 ] || return;
-	for file in "${LIBUI_LOG_FILE[@]}"; do
+	for file in $LIBUI_LOG_FILE; do
+		[ -z "$file" ] && continue;
 		dir=$(dirname $file)
 		mkdir -p $dir || die_error "Cannot create log directory $dir"
 		str="[LOG] `date +"%Y-%m-%d %H:%M:%S"` $@"
@@ -130,12 +131,14 @@ log ()
 debug ()
 {
 	[ "$LIBUI_DEBUG" = "1" ] || return;
+	[ -n "$1" ] || die error "you must specify at least one (non-empty) debug category"
+	[ -n "$2" ] || die_error "debug \$2 cannot be empty"
 	for cat in $1
 	do
 		check_is_in $cat "${LIBUI_DEBUG_CATEGORIES[@]}" || die_error "debug \$1 contains a value ($cat) which is not a valid debug category"
 	done
-	[ -n "$2" ] || die_error "debug \$2 cannot be empty"
-	for file in "${LIBUI_LOG_FILE[@]}"; do
+	for file in $LIBUI_LOG_FILE; do
+		[ -z "$file" ] && continue;
 		dir=$(dirname $file)
 		mkdir -p $dir || die_error "Cannot create log directory $dir"
 		str="[DEBUG $1 ] $2"
@@ -244,6 +247,22 @@ ask_string ()
 }
 
 
+# ask for multiple strings.
+# this function is not done yet.
+# TODO: how to return data back to user? a variable with one each line a value?
+# TODO: dialog functionality not good
+# $1 question/title
+# $2 exitcode to use when one of the strings is empty and there was no default, or default was ignored (1 default)
+# [ $3 label, $4 default value , .. ]
+# returns 1 if the user cancelled, 0 otherwise
+ask_string_multiple ()
+{
+	[ -z "$1" ] && die_error "ask_string_multiple needs a question!"
+	[ `type -t _${LIBUI_UI}_ask_string_multiple` == function ] || die_error "_${LIBUI_UI}_ask_string_multiple is not a function"
+	_${LIBUI_UI}_ask_string_multiple "$@"
+}
+
+
 # ask a yes/no question.
 # $1 question
 # $2 default answer yes/no (optional)
@@ -303,7 +322,7 @@ _dia_notify ()
 }
 
 
-_dia_infofy ()
+_dia_inform ()
 {
 	str="$1"
 	if [ "$2" != 0 ]
@@ -438,6 +457,33 @@ _dia_ask_string ()
 }
 
 
+_dia_ask_string_multiple ()
+{
+    MAXRESPONSE=0
+    formtitle="$1"
+    exitcode="${2:-1}"
+    shift 2
+
+    formitems=""
+    line=1
+	unset m; i=0
+    while [ -n "$1" ]
+    do
+        [ -z "$2" ] && die_error "No default value for $1"
+        # format: Label X Y Value X Y display-size value-size
+		words=("$1" $line 1 "$2" $line 20 20 $MAXRESPONSE)
+		for w in "${words[@]}"; do
+			formitems[i++]=$w; formitems[i++]=""
+		done
+		let line++
+
+        formitems="$formitems $this_item"
+        shift 2
+    done
+    _dia_dialog --form "$formtitle" 15 50 0 "${formitems[@]}"
+}
+
+
 
 _dia_ask_yesno ()
 {
@@ -491,7 +537,7 @@ _cli_notify ()
 }
 
 
-_cli_infofy ()
+_cli_inform ()
 {
 	echo -e "$1"
 }
@@ -641,6 +687,21 @@ _cli_ask_string ()
 }
 
 
+_cli_ask_string_multiple () {
+	echo "$1"
+	exitcode=${2:-1}
+	shift 2
+
+	ANSWER_VALUES=()
+	i=0
+	while [ $# -gt 0 ]; do
+		_cli_ask_string "$1" "$2" $exitcode
+		ANSWER_VALUES[$i]="$ANSWER_STRING"
+		let i++
+		shift 2
+	done
+}
+
 _cli_ask_yesno ()
 {
 	[ -z "$2"    ] && echo -n "$1 (y/n): "
